// SMETS AND WOUTERS (2007) MODEL CODE [ADJUSTED TO TAKE INTO ACCOUNT STOCHASTIC DETRENDING AS IN NEGRO & SCHORFHEIDE (2012)]
//**According to Negro's matlab code - 'eqcond803.m'
//**after the dynare forum feedback

//BLOCK-1//##//##//##//##//#/*VARIABLES DECLARATION*\#\\##\\##\\##\\##\\##\

var   labobs robs pinfobs dy dc dinve dw  ewma epinfma  zcapf rkf kf pkf    cf invef yf labf wf rrf mc zcap rk k pk    c inve y lab pinf w r b g qs  ms  spinf sw kpf kp     zgrowth zdgrowth;    //adjusted - deleted 'a' 
varexo ez eb eg  eqs  em  epinf ew;                                                                      //adjusted for SN detrending (replaced 'ea' with 'ez' variable)
parameters curvw cgy curvp constelab cmaw cmap calfa 
czcap cbeta csadjcost ctou csigma chabb cfc 
cindw cprobw cindp cprobp csigl clandaw 
crpi crdy cry crr 
crhoz crhob crhog crhoqs crhoms crhopinf crhow 
cg cgamma clandap cbetabar cr cpie crk cw cikbar cik clk cky ciy ccy crkky cwlc cwly ;           //adjusted - replacing 'crhoa' with 'crhoz'
//constepinf conster ctrend

//BLOCK-2//##//##//##//#//#/*PARAMETER INITIALIZATION*\#\\#\\##\\##\\##\\##

// FIXED PARAMETERS
ctou=     0.025;
clandaw=  1.5;
cg=       0.18;
curvp=    10;
curvw=    10;

// ESTIMATED PARAMETERS INITIALISATION
cgamma=   1.004;
cbeta=    0.9995;
cpie=     1.005;

crhoz=    0.0;                                                                                           //adjusted. replaced with 'crhoz' below
crhob=    0.5799;
crhog=    0.9957;
crhoqs=   0.7165;
crhoms=   0;
crhopinf= 0;
crhow=    0;
cmap =    0;
cmaw =    0;

csadjcost=6.0144;
csigma=   1.5;
chabb=    0.6361;    
cprobw=   0.8087;
csigl=    1.9423;
cprobp=   0.6;
cindw=    0.3243;
cindp=    0.47;
czcap=    0.2696;
cfc=      1.5;
crpi=     1.488;
crr=      0.8762;
cry=      0.0593;
crdy=     0.2347;
cgy=      0.51;
calfa=    0.24;


//BLOCK-3//##//##//##//##//#//#/*STEADY STATE*\#\\#\\##\\##\\##\\##\\##\\

// DERIVED FROM THE STEADY STATE~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~(ADJUSTED)

clandap=cfc;
cbetabar=cbeta*exp(cgamma*(1-csigma));                                                                   //adjusted
cr=(cpie/cbeta)*exp(cgamma);                                                                             //adjusted (eq. 1.2.16 SN-Appendix)
crk=(1/cbeta)*(exp(cgamma)) - (1-ctou);                                                                  //adjusted (eq. 1.2.43 SN-Apx)
cw = ((1/(1+clandap))*(calfa^calfa)*((1-calfa)^(1-calfa))*(crk^(-calfa)))^(1/(1-calfa));                 //adjusted (eq. 1.2.44 SN-Apx)
cikbar=(1-(1-ctou)/exp(cgamma));                                                                         //adjusted (eq. 1.2.48)
cik=(1-(1-ctou)/exp(cgamma))*exp(cgamma);                                                                //adjusted (sub eq. 1.2.47 into 1.2.48)
clk=((1-calfa)/calfa)*(crk/cw);                                                                          //adjusted (eq. 1.2.45)
cky=(calfa/(1+clandap))*((1/cbeta)*exp(csigma)-(1-ctou))^(-1);                                           //adjusted (eq. 1.2.59)
ciy=cik*cky;
ccy=(1/cg)-ciy;                                                                                          //adjusted (eq. 1.2.50)
crkky=crk*cky;
cwlc=((1-calfa)/calfa)*crk*cky/ccy;                                                                      //instead of w*(h) in SW eq. 2, it has w* as in the SN paper eq. 3
cwly=(1-calfa)/(1+clandap);

//ctrend=(cgamma-1)*100;
//conster=(cr-1)*100;
//constepinf=(cpie-1)*100;
constelab=0;


//BLOCK-4//##//##//##//##//##//#/*MODEL BLOCK*\#\\##\\##\\##\\##\\##\\##\\

model(linear); 

//#usmodel_stst;
//@#include "usmodel_stst.m"

// FLEXIBLE ECONOMY~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#(ADJUSTED)
	      cf = (chabb*exp(-cgamma))/(1+chabb*exp(-cgamma))*(cf(-1)-zgrowth) + (1/(1+chabb*exp(-cgamma)))*cf(+1) + (1/(1-calfa))*(crhoz-1)*(1/(1+chabb*exp(-cgamma)))*zdgrowth + ((csigma-1)*cwlc/(csigma*(1+chabb*exp(-cgamma))))*(labf-labf(+1)) - ((1-chabb*exp(-cgamma))/(csigma*(1+chabb*exp(-cgamma))))*rrf + b ;
	      invef = (1/(1+cbetabar))*(invef(-1) + cbetabar*invef(+1) - zgrowth + (1/(1-calfa))*(crhoz-1)*cbetabar*zdgrowth + (1/(csadjcost*exp(2*cgamma)))*pkf ) +qs ;
          pkf = - rrf + ((csigma*(1+chabb*exp(-cgamma)))/(1-chabb*exp(-cgamma)))*b + (crk/(crk+(1-ctou)))*rkf(+1) +  ((1-ctou)/(crk+(1-ctou)))*pkf(+1);
	      yf = cfc*( calfa*kf+(1-calfa)*labf ) + ((cfc-1)/(1-calfa))*zdgrowth;
	      kf =  kpf(-1) + zcapf - zgrowth;
	      zcapf =  ((1-czcap)/czcap)*rkf  ;
	      kpf =  (1-cikbar)*kpf(-1) - (1-cikbar)*zgrowth + cikbar*invef + cikbar*csadjcost*exp(2*cgamma)*(1+cbetabar)*qs;
	      wf =  - calfa*labf + calfa*kf;                            
	      rkf =  wf + labf - kf ;
	      wf = csigl*labf + (1/(1-chabb*exp(-cgamma)))*cf - (chabb*exp(-cgamma))/(1-chabb*exp(-cgamma))*cf(-1) + ((chabb*exp(-cgamma))/(1-chabb*exp(-cgamma)))*zgrowth ;
	      yf = ccy*cf+ ciy*invef + cg*g  +  crkky*zcapf;

// STICKY PRICE - WAGE ECONOMY~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~(ADJUSTED)
          
          //consumption euler equation
	      c = ((chabb*exp(-cgamma))/(1+chabb*exp(-cgamma)))*(c(-1)-zgrowth) + (1/(1+chabb*exp(-cgamma)))*c(+1) + (1/(1-calfa))*(crhoz-1)*(1/(1+chabb*exp(-cgamma)))*zdgrowth + (((csigma-1)*cwlc)/(csigma*(1+chabb*exp(-cgamma))))*(lab-lab(+1)) - ((1-chabb*exp(-cgamma))/(csigma*(1+chabb*exp(-cgamma))))*(r-pinf(+1)) + b ;     //adjusted for SN detrending (added zgrowth variable)

          //investment euler equation
	      inve = (1/(1+cbetabar))*(inve(-1) + cbetabar*inve(+1) - zgrowth + (1/(1-calfa))*(crhoz-1)*cbetabar*zdgrowth + (1/(csadjcost*exp(2*cgamma)))*pk ) +qs ;                                                                                                                                         //adjusted for SN detrending (added zgrowth variable)

          //value of capital
          pk = - r + pinf(1) + ((csigma*(1+chabb*exp(-cgamma)))/(1-chabb*exp(-cgamma)))*b + (crk/(crk+(1-ctou)))*rk(+1) + ((1-ctou)/(crk+(1-ctou)))*pk(+1);

          //aggregate production function
	      y = cfc*( calfa*k+(1-calfa)*lab ) + ((cfc-1)/(1-calfa))*zdgrowth;

          //capital utilization
	      k =  kp(-1) + zcap - zgrowth;

          //rental rate of capital
	      zcap =  ((1-czcap)/czcap)*rk ;

          //evolution of capital
	      kp =  (1-cikbar)*kp(-1) - (1-cikbar)*zgrowth + cikbar*inve + cikbar*csadjcost*exp(2*cgamma)*(1+cbetabar)*qs ;                                                                                                                                                       //adjusted

          //price mark-up
	      mc =  w + calfa*lab - calfa*k;            

          //phillips curve
	      pinf =  (1/(1+cbetabar*cindp))*(cbetabar*pinf(+1) + cindp*pinf(-1)) + (((1-cprobp)*(1-cbetabar*cprobp))/(cprobp*((cfc-1)*curvp+1))*1/(1+cbetabar*cindp))*mc + spinf ; 

          //rental rate of capital
	      rk =  w + lab - k ;

          //evolution of wages
	      w =  (1/(1+cbetabar))*w(-1)
               -(1/(1+cbetabar))*zgrowth
               +(cbetabar/(1+cbetabar))*w(+1)
               +(1/(1-calfa))*(crhoz-1)*(cbetabar/(1+cbetabar))*zdgrowth
               -(cindw/(1+cbetabar))*pinf(-1)
               -((1+cbetabar*cindw)/(1+cbetabar))*pinf
               +((cbetabar)/(1+cbetabar))*pinf(+1)
               +(((1-cprobw)*(1-cbetabar*cprobw))/(cprobw*((clandaw-1)*curvw+1))*1/(1+cbetabar))*
               (csigl*lab + (1/(1-chabb*exp(-cgamma)))*c - ((chabb*exp(-cgamma))/(1-chabb*exp(-cgamma)))*c(-1) + ((chabb*exp(-cgamma))/(1-chabb*exp(-cgamma)))*zgrowth - w) 
               + 1*sw ;                                                                                                                                                                                                                                                         //adjusted for SN detrending (added zgrowth variable)

          //monetary policy rule
	      r =  crpi*(1-crr)*pinf
               +cry*(1-crr)*(y-yf)     
               +crdy*(y-yf-y(-1)+yf(-1))
               +crr*r(-1)
               +ms  ;

          //resource constraint
	      y = ccy*c + ciy*inve + cg*g + crkky*zcap;

          //exogenous processes
          zgrowth = (1/(1-calfa))*(crhoz-1)*zdgrowth(-1)+(1/(1-calfa))*ez;                               //productivity/technology process         //added - exogenous process of SW deleted. replaced with the process in NS (2012) Appendix (eq 1.1.46)
          zdgrowth = crhoz*zdgrowth(-1) + ez;                                                            //added
          b = crhob*b(-1) + eb;                                                                          //risk premium process
	      g = crhog*g(-1) + eg + cgy*ez;                                                                 //exogenous spending process                                                                  
	      qs = crhoqs*qs(-1) + eqs;                                                                      //inv. technology process
	      ms = crhoms*ms(-1) + em;                                                                       //monetary policy process
	      spinf = crhopinf*spinf(-1) + epinfma - cmap*epinfma(-1);                                       //price markup process
            epinfma=epinf;
	      sw = crhow*sw(-1) + ewma - cmaw*ewma(-1) ;                                                     //wage markup process
            ewma=ew; 

// MEASUREMENT EQUATIONS~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~(ADJUSTED)
//dy=y-y(-1)+zgrowth+ctrend;
//dc=c-c(-1)+zgrowth+ctrend;
//dinve=inve-inve(-1)+zgrowth+ctrend;
//dw=w-w(-1)+zgrowth+ctrend;
//pinfobs = 1*(pinf) + constepinf;
//robs =    1*(r) + conster;
//labobs = lab + constelab;

dy=100*(y-y(-1)+zgrowth)+cgamma;
dc=100*(c-c(-1)+zgrowth)+cgamma;
dinve=100*(inve-inve(-1)+zgrowth)+cgamma;
dw=100*(w-w(-1)+zgrowth)+cgamma;
pinfobs = 100*(pinf) + cpie;
robs =    100*(r) + cr;
labobs = 100*lab + constelab;

end; 


//BLOCK-5//##//##//##//##//##//##/*SHOCKS*\##\\##\\##\\##\\##\\##\\##\\##\\

shocks;                                                                                                  
var ez;                                                                                                  //replaced with the shock for 'ez'
stderr 0.4618;
var eb;
stderr 1.8513;
var eg;
stderr 0.6090;
var eqs;
stderr 0.6017;
var em;
stderr 0.2397;
var epinf;
stderr 0.1455;
var ew;
stderr 0.2089;
end;


//BLOCK-6//##//##//##//##//##//#/*ESTIMATIONS*\#\\##\\##\\##\\##\\##\\##\\

estimated_params;

//PRIOR ESTIMATES
// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF                                           
stderr ez,0.4618,0.01,3,INV_GAMMA_PDF,0.1,2;                                                             //replaced with the equation for 'ez' 
stderr eb,0.1818513,0.025,5,INV_GAMMA_PDF,0.1,2;
stderr eg,0.6090,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr eqs,0.46017,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr em,0.2397,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr epinf,0.1455,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr ew,0.2089,0.01,3,INV_GAMMA_PDF,0.1,2;
crhoz,.9676 ,.01,.9999,BETA_PDF,0.5,0.20;                                                                //replaced with the equation for 'crhoz' 
crhob,.2703,.01,.9999,BETA_PDF,0.5,0.20;
crhog,.9930,.01,.9999,BETA_PDF,0.5,0.20;
crhoqs,.5724,.01,.9999,BETA_PDF,0.5,0.20;
crhoms,.3,.01,.9999,BETA_PDF,0.5,0.20;
crhopinf,.8692,.01,.9999,BETA_PDF,0.5,0.20;
crhow,.9546,.001,.9999,BETA_PDF,0.5,0.20;
cmap,.7652,0.01,.9999,BETA_PDF,0.5,0.2;
cmaw,.8936,0.01,.9999,BETA_PDF,0.5,0.2;
csadjcost,6.3325,2,15,NORMAL_PDF,4,1.5;
csigma,1.2312,0.25,3,NORMAL_PDF,1.50,0.375;
chabb,0.7205,0.001,0.99,BETA_PDF,0.7,0.1;
cprobw,0.7937,0.3,0.95,BETA_PDF,0.5,0.1;
csigl,2.8401,0.25,10,NORMAL_PDF,2,0.75;
cprobp,0.7813,0.5,0.95,BETA_PDF,0.5,0.10;
cindw,0.4425,0.01,0.99,BETA_PDF,0.5,0.15;
cindp,0.3291,0.01,0.99,BETA_PDF,0.5,0.15;
czcap,0.2648,0.01,1,BETA_PDF,0.5,0.15;
cfc,1.4672,1.0,3,NORMAL_PDF,1.25,0.125;
crpi,1.7985,1.0,3,NORMAL_PDF,1.5,0.25;
crr,0.8258,0.5,0.975,BETA_PDF,0.75,0.10;
cry,0.0893,0.001,0.5,NORMAL_PDF,0.125,0.05;
crdy,0.2239,0.001,0.5,NORMAL_PDF,0.125,0.05;
//constepinf,0.7,0.1,2.0,GAMMA_PDF,0.625,0.1;//20;
constelab,1.2918,-10.0,10.0,NORMAL_PDF,0.0,2.0;
//ctrend,0.3982,0.1,0.8,NORMAL_PDF,0.4,0.10;
cgy,0.05,0.01,2.0,NORMAL_PDF,0.5,0.25;
calfa,0.24,0.01,1.0,NORMAL_PDF,0.3,0.05;
end;

//OBSERVED VARIABLES
varobs dy dc dinve labobs pinfobs dw robs;
identification(prior_mc=250);

//ESTIAMTION COMMAND
 estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=9,mode_check,first_obs=147,presample=4,lik_init=2,prefilter=0,mh_replic=20000,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);//,bayesian_irf,irf=20,moments_varendo,filtered_vars,smoother);
// estimation(datafile=eudataset,mode_compute=6,first_obs=1,presample=4,lik_init=2,prefilter=0,mh_replic=20000,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2,bayesian_irf,irf=20,moments_varendo,filtered_vars,smoother);

//IRF
// Generate IRfunctions and Moments of output, inflation and interest rate
//   stoch_simul(irf=20) dy pinfobs robs ;


// Calculation the marginal likelihood with training period (40 observations between '56 and '65)
// estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_31_mode  ,first_obs=31,nobs=200,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
// estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_3144_mode,first_obs=31,nobs=44,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);

// Outcomes for subperiods (66:1-79:2 and 84:1-04:4)
// estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_7158_mode,first_obs=71,nobs=58,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//   stoch_simul(irf=20) dy pinfobs robs ;
// estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_143_mode,first_obs=143,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//   stoch_simul(irf=20) dy pinfobs robs ;

// Recursive estimates for forecasting: starting at 90:1
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_100_mode,first_obs=71,nobs=100,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_104_mode,first_obs=71,nobs=104,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_108_mode,first_obs=71,nobs=108,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_112_mode,first_obs=71,nobs=112,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_116_mode,first_obs=71,nobs=116,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_120_mode,first_obs=71,nobs=120,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_124_mode,first_obs=71,nobs=124,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_128_mode,first_obs=71,nobs=128,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_132_mode,first_obs=71,nobs=132,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_136_mode,first_obs=71,nobs=136,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_140_mode,first_obs=71,nobs=140,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_144_mode,first_obs=71,nobs=144,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_148_mode,first_obs=71,nobs=148,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_152_mode,first_obs=71,nobs=152,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//  estimation(optim=('MaxIter',200),datafile=usmodel_data,mode_compute=0,mode_file=usmodel_hist_dsge_f19_7_71_156_mode,first_obs=71,nobs=156,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);


 